//---------------------------------------------------------------------
//  Copyright (C) Microsoft Corporation.  All rights reserved.
// 
// THIS CODE AND INFORMATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
//
//---------------------------------------------------------------------

using System;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using System.Windows.Forms.Design.Behavior;

namespace DesignSurfaceExtenderLibrary.Extensions
{
    /// <summary>
    /// A default implementation of an action list provided
    /// by an <see cref="DesignSurfaceExtension"/>.
    /// </summary>
    public class ExtensionActionList : DesignerActionList
    {   
        /// <summary>
        /// Backs the <see cref="Extension"/> property.
        /// </summary>
        private DesignSurfaceExtension extensionValue;

        /// <summary>
        /// The name of the action list's property indicating
        /// whether the <see cref="DesignSurfaceExtension"/> is active.
        /// </summary>
        private static string propertyNameValue = "Enabled";

        /// <summary>
        /// The display text which appears on the smart tag panel. 
        /// </summary>
        /// <remarks>The display text is type-specific. Deriving types 
        /// must assign this string.
        /// </remarks>
        protected string displayTextValue;
        
        /// <summary>
        /// This constructor is private to prevent instantiation.
        /// </summary>
        /// <param name="component">The related <see cref="DesignSurfaceExtender"/> component.</param>
        private ExtensionActionList( IComponent component )
            : base( component )
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ExtensionActionList"/> class.
        /// </summary>
        /// <param name="extension">The <see cref="DesignSurfaceExtension"/> which provides this action list.</param>
        protected ExtensionActionList( DesignSurfaceExtension extension )
            : base( null )
        {
            this.extensionValue = extension;
        }

        /// <summary>
        /// Returns the collection of <see cref="DesignerActionItem"/> objects 
        /// contained in the list. 
        /// </summary>
        /// <returns>The type-specific list containing a <see cref="DesignerActionPropertyItem"/>
        /// binding defined by <see cref="propertyNameValue"/> and <see cref="displayTextValue"/>
        /// fields.</returns>
        /// <remarks>Deriving classes must assign the <see cref="propertyNameValue"/> and
        /// <see cref="displayTextValue"/> fields.
        /// </remarks>
        public override DesignerActionItemCollection GetSortedActionItems()
        {
            DesignerActionItemCollection items =
                new DesignerActionItemCollection();

            items.Add( new DesignerActionPropertyItem(
                propertyNameValue,
                displayTextValue ) );

            return items;
        }

        /// <summary>
        /// Gets the <see cref="DesignSurfaceExtension"/> which provides this action list.
        /// </summary>
        public DesignSurfaceExtension Extension
        {
            get
            {
                return this.extensionValue;
            }        
        }

        /// <summary>
        /// Gets or sets a value indicating whether the <see cref="DesignSurfaceExtension"/> is
        /// active.
        /// </summary>
        /// <remarks>Setting the <see cref="Enabled"/> property causes 
        /// the <see cref="Adorner"/> to repaint.
        /// </remarks>
        public bool Enabled
        {
            get
            {
                return this.Extension.Enabled;
            }

            set
            {
                this.Extension.Enabled = value;

                this.Extension.Invalidate();
            }
        }
    }
}
